@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2022 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2022 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -50,6 +50,9 @@
 
 #include "../../deadbeef.h"
 #include "artwork_flac.h"
+#ifdef USE_OGG
+#include "artwork_ogg.h"
+#endif
 #include "albumartorg.h"
 #include "artwork.h"
 #include "artwork_internal.h"
@@ -662,7 +665,7 @@ web_lookups (const char *cache_path, ddb_cover_info_t *cover) {
     if (!cache_path) {
         return -1;
     }
-#if USE_VFS_CURL
+#ifdef USE_VFS_CURL
     if (artwork_enable_lfm) {
         if (!fetch_from_lastfm (cover->priv->artist, cover->priv->album, cache_path)) {
             cover->image_filename = strdup (cache_path);
@@ -904,10 +907,18 @@ process_query (ddb_cover_info_t *cover) {
             cover->cover_found = 1;
             return;
         }
+
+#ifdef USE_OGG
+        trace ("trying to load artwork from ogg tag for %s\n", cover->priv->filepath);
+        if (!ogg_extract_art (cover)) {
+            _consume_blob (cover, simplified_cache ? cover->priv->album_cache_path : cover->priv->track_cache_path);
+            cover->cover_found = 1;
+            return;
+        }
+#endif
     }
 
 #ifdef USE_VFS_CURL
-
     // Don't do anything if all web lookups are off
     if (!artwork_enable_wos
         && !artwork_enable_lfm
@@ -1075,7 +1086,7 @@ _setup_tf_once() {
             album_tf = deadbeef->tf_compile ("%album%");
         }
         if (!artist_tf) {
-            artist_tf = deadbeef->tf_compile ("%artist%");
+            artist_tf = deadbeef->tf_compile ("$itematindex(0,%artist%)");
         }
         if (!title_tf) {
             title_tf = deadbeef->tf_compile ("%title%");
@@ -1379,6 +1390,7 @@ _init_cover_metadata(ddb_cover_info_t *cover, ddb_playItem_t *track) {
     ctx._size = sizeof (ddb_tf_context_t);
     ctx.it = track;
 
+#ifdef USE_VFS_CURL
     if (artwork_enable_wos && strlen (cover->priv->filepath) > 3 && !strcasecmp (cover->priv->filepath + strlen (cover->priv->filepath) - 3, ".ay")) {
         strcpy (cover->priv->artist, "AY Music");
         deadbeef->tf_eval (&ctx, title_tf, cover->priv->album, sizeof (cover->priv->album));
@@ -1390,7 +1402,9 @@ _init_cover_metadata(ddb_cover_info_t *cover, ddb_playItem_t *track) {
         }
         strcpy (cover->priv->title, cover->priv->album);
     }
-    else {
+    else
+#endif
+    {
         deadbeef->tf_eval (&ctx, album_tf, cover->priv->album, sizeof (cover->priv->album));
         deadbeef->tf_eval (&ctx, artist_tf, cover->priv->artist, sizeof (cover->priv->artist));
         deadbeef->tf_eval (&ctx, title_tf, cover->priv->title, sizeof (cover->priv->title));
@@ -1437,7 +1451,7 @@ cover_get (ddb_cover_query_t *query, ddb_cover_callback_t callback) {
         }
 
         /* Process this query, hopefully writing a file into cache */
-        ddb_cover_info_t *cover = sync_cover_info_alloc();
+        __block ddb_cover_info_t *cover = sync_cover_info_alloc();
 
         _init_cover_metadata(cover, query->track);
 
@@ -1454,10 +1468,19 @@ cover_get (ddb_cover_query_t *query, ddb_cover_callback_t callback) {
         }
 
         // check the cache
-        ddb_cover_info_t *cached_cover = cover_cache_find (cover);
-        if (cached_cover) {
-            cached_cover->priv->timestamp = time(NULL);
-            cover = cached_cover;
+        __block int found_in_cache = 0;
+        dispatch_sync(sync_queue, ^{
+            ddb_cover_info_t *cached_cover = cover_cache_find (cover);
+            if (cached_cover) {
+                found_in_cache = 1;
+                cached_cover->priv->timestamp = time(NULL);
+                cover_info_release(cover);
+                cover = cached_cover;
+            }
+        });
+
+
+        if (found_in_cache) {
             _execute_callback (callback, cover, query);
         }
         else {
@@ -1949,7 +1972,7 @@ ddb_artwork_plugin_t plugin = {
     .plugin.plugin.descr = "Loads album artwork from embedded tags, local directories, or internet services",
     .plugin.plugin.copyright =
         "DeaDBeeF -- the music player\n"
-        "Copyright (C) 2009-2021 Alexey Yakovenko and other contributors\n"
+        "Copyright (C) 2009-2021 Oleksiy Yakovenko and other contributors\n"
         "\n"
         "This software is provided 'as-is', without any express or implied\n"
         "warranty.  In no event will the authors be held liable for any damages\n"
