@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2015 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2015 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -397,17 +397,6 @@ static const int close_btn_left_offs = 8;
         return;
     }
 
-    int tab_playing = -1;
-    DB_playItem_t *playing = deadbeef->streamer_get_playing_track ();
-    if (playing) {
-        ddb_playlist_t *plt = deadbeef->pl_get_playlist (playing);
-        if (plt) {
-            tab_playing = deadbeef->plt_get_idx (plt);
-            deadbeef->plt_unref (plt);
-        }
-        deadbeef->pl_item_unref (playing);
-    }
-
     int need_draw_moving = 0;
     int idx;
     int widths[cnt];
@@ -684,6 +673,7 @@ static const int close_btn_left_offs = 8;
         menu.clickPoint = coord;
         menu.delegate = self;
         menu.renamePlaylistDelegate = self;
+        menu.deletePlaylistDelegate = self;
         menu.autoenablesItems = YES;
 
         ddb_playlist_t *plt = deadbeef->plt_get_for_idx ((int)_clickedTabIndex);
