@@ -2,8 +2,8 @@
 //  PlaylistContentView.m
 //  DeaDBeeF
 //
-//  Created by Alexey Yakovenko on 2/1/20.
-//  Copyright © 2020 Alexey Yakovenko. All rights reserved.
+//  Created by Oleksiy Yakovenko on 2/1/20.
+//  Copyright © 2020 Oleksiy Yakovenko. All rights reserved.
 //
 
 #import "PinnedGroupTitleView.h"
@@ -41,8 +41,7 @@ static int grouptitleheight = 22;
 @property (nonatomic) NSMutableArray<PlaylistGroup *> *groups;
 @property (nonatomic,readwrite) int grouptitle_height;
 @property (nonatomic) int groups_build_idx;
-@property (nonatomic) int fullwidth;
-@property (nonatomic) int fullheight;
+@property (nonatomic) CGSize contentSize;
 @property (nonatomic) int scroll_direction;
 @property (nonatomic) int scroll_pointer_y;
 
@@ -351,8 +350,6 @@ static int grouptitleheight = 22;
 }
 
 - (void)drawListView:(NSRect)dirtyRect {
-    [self groupCheck];
-
     CGFloat clip_y = dirtyRect.origin.y;
     CGFloat clip_h = dirtyRect.size.height;
 
@@ -470,8 +467,8 @@ static int grouptitleheight = 22;
     [self drawListView:dirtyRect];
 
     // draw rows below the real list
-    if ([self fullheight] < dirtyRect.origin.y + dirtyRect.size.height) {
-        int y = [self fullheight];
+    if (_contentSize.height < dirtyRect.origin.y + dirtyRect.size.height) {
+        int y = _contentSize.height;
         int ii = [self.dataModel rowCount]+1;
         while (y < dirtyRect.origin.y + dirtyRect.size.height) {
             if (y + rowheight >= dirtyRect.origin.y) {
@@ -660,10 +657,6 @@ static int grouptitleheight = 22;
 
 - (void)mouseDragged:(NSEvent *)event
 {
-    NSPoint dragLocation;
-    dragLocation=[self convertPoint:[event locationInWindow]
-                           fromView:nil];
-
     [self listMouseDragged:event];
 
     // support automatic scrolling during a drag
@@ -1160,8 +1153,8 @@ static int grouptitleheight = 22;
 
     [self freeGroups];
 
-    _fullwidth = 0;
-    _fullheight = 0;
+    CGSize contentSize = CGSizeZero;
+
     PlaylistGroup *grp = NULL;
 
     NSString *str;
@@ -1172,7 +1165,7 @@ static int grouptitleheight = 22;
         if ([self.delegate columnMinHeight:c] && [self.delegate columnWidth:c] > min_height) {
             min_height = [self.delegate columnGroupHeight:c];
         }
-        _fullwidth += [self.delegate columnWidth:c];
+        contentSize.width += [self.delegate columnWidth:c];
     }
 
     _grouptitle_height = grouptitleheight;
@@ -1194,7 +1187,7 @@ static int grouptitleheight = 22;
                 if (grp->height - _grouptitle_height < min_height) {
                     grp->height = min_height + _grouptitle_height;
                 }
-                _fullheight += grp->height;
+                contentSize.height += grp->height;
             }
 
             [self.groups addObject:newgroup];
@@ -1222,9 +1215,10 @@ static int grouptitleheight = 22;
         if (grp->height - _grouptitle_height < min_height) {
             grp->height = min_height + _grouptitle_height;
         }
-        _fullheight += grp->height;
+        contentSize.height += grp->height;
     }
-    [self updateContentFrame];
+
+    [self updateContentFrame:contentSize];
 }
 
 - (void)groupCheck {
@@ -1374,28 +1368,36 @@ static int grouptitleheight = 22;
     [self scrollPoint:NSMakePoint(vis.origin.x, verticalPosition)];
 }
 
-- (void)updateContentFrame {
-    _fullwidth = 0;
-    for (DdbListviewCol_t c = [self.delegate firstColumn]; c != [self.delegate invalidColumn]; c = [self.delegate nextColumn:c]) {
-        _fullwidth += [self.delegate columnWidth:c];
-    }
-
+- (void)updateContentFrame:(CGSize)size {
     if (!self.widthConstraint) {
-        self.widthConstraint = [self.widthAnchor constraintGreaterThanOrEqualToConstant:_fullwidth];
-        self.heightConstraint = [self.heightAnchor constraintGreaterThanOrEqualToConstant:_fullheight];
+        self.widthConstraint = [self.widthAnchor constraintGreaterThanOrEqualToConstant:size.width];
+        self.heightConstraint = [self.heightAnchor constraintGreaterThanOrEqualToConstant:size.height];
         self.widthConstraint.priority = NSLayoutPriorityDefaultHigh;
         self.heightConstraint.priority = NSLayoutPriorityDefaultHigh;
         self.widthConstraint.active = YES;
         self.heightConstraint.active = YES;
     }
     else {
-        self.widthConstraint.constant = _fullwidth;
-        self.heightConstraint.constant = _fullheight;
+        if (_contentSize.width != size.width) {
+            self.widthConstraint.constant = size.width;
+        }
+        if (_contentSize.height != size.height) {
+            self.heightConstraint.constant = size.height;
+        }
     }
 
+    _contentSize = size;
     [self updatePinnedGroup];
 }
 
+- (void)columnsDidChange {
+    CGSize contentSize = _contentSize;
+    contentSize.width = 0;
+    for (DdbListviewCol_t c = [self.delegate firstColumn]; c != [self.delegate invalidColumn]; c = [self.delegate nextColumn:c]) {
+        contentSize.width += [self.delegate columnWidth:c];
+    }
+}
+
 - (NSInteger)getScrollFocusGroupAndOffset:(CGFloat *)offset {
     if (self.groups.count == 0) {
         *offset = 0;
@@ -1405,14 +1407,12 @@ static int grouptitleheight = 22;
     NSRect rect = self.enclosingScrollView.documentVisibleRect;
     CGFloat clip_y = rect.origin.y;
 
-    int idx = 0;
     int grp_y = 0;
     NSUInteger groupIndex = 0;
 
     while (groupIndex < self.groups.count && grp_y < clip_y) {
         PlaylistGroup *grp = self.groups[groupIndex];
         grp_y += grp->height;
-        idx += grp->num_items;
         groupIndex++;
     }
 
