@@ -2,8 +2,8 @@
 //  PlaylistLocalDragDropHolder.h
 //  DeaDBeeF
 //
-//  Created by Alexey Yakovenko on 2/1/20.
-//  Copyright © 2020 Alexey Yakovenko. All rights reserved.
+//  Created by Oleksiy Yakovenko on 2/1/20.
+//  Copyright © 2020 Oleksiy Yakovenko. All rights reserved.
 //
 
 #import <Foundation/Foundation.h>
@@ -15,7 +15,6 @@ NS_ASSUME_NONNULL_BEGIN
 
 @property (nonatomic) NSInteger playlistIdx;
 @property (nonatomic) NSArray *itemsIndices;
-@property (nonatomic,readonly) int count;
 
 - (instancetype)init NS_UNAVAILABLE; 
 - (instancetype)initWithSelectedItemsOfPlaylist:(ddb_playlist_t *)playlist;
