@@ -2,8 +2,8 @@
 //  TrackContextMenu.m
 //  DeaDBeeF
 //
-//  Created by Alexey Yakovenko on 7/27/20.
-//  Copyright © 2020 Alexey Yakovenko. All rights reserved.
+//  Created by Oleksiy Yakovenko on 7/27/20.
+//  Copyright © 2020 Oleksiy Yakovenko. All rights reserved.
 //
 
 #include <sys/stat.h>
@@ -37,6 +37,7 @@ extern DB_functions_t *deadbeef;
 @property (nonatomic) NSMenuItem *rgScanAsAlbumsItem;
 @property (nonatomic) NSMenuItem *rgRemoveInformationItem;
 
+@property (nonatomic) NSMenuItem *showInFinderItem;
 @property (nonatomic) NSMenuItem *deleteFromDiskItem;
 
 @property (nonatomic) NSMenuItem *pluginActionsSeparatorItem;
@@ -140,6 +141,8 @@ extern DB_functions_t *deadbeef;
     self.convertItem = [self addItemWithTitle:@"Convert" action:@selector(convertSelection) keyEquivalent:@""];
     self.convertItem.target = self;
 
+    self.showInFinderItem = [self addItemWithTitle:@"Show in Finder" action:@selector(showInFinder) keyEquivalent:@""];
+    self.showInFinderItem.target = self;
     self.deleteFromDiskItem = [self addItemWithTitle:@"Delete from Disk" action:@selector(deleteFromDisk) keyEquivalent:@""];
     self.deleteFromDiskItem.target = self;
 
@@ -159,6 +162,7 @@ extern DB_functions_t *deadbeef;
         selected = tracks[0];
     }
 
+    self.showInFinderItem.enabled = selected_count != 0;
     self.deleteFromDiskItem.enabled = selected_count != 0;
 
     if ([self addPluginActionItemsForSelectedTrack:selected selectedCount:selected_count actionContext:actionContext]) {
@@ -375,17 +379,21 @@ static void _warningMessageForCtx (ddbDeleteFromDiskController_t ctl, ddb_action
 static int
 _deleteFile (ddbDeleteFromDiskController_t ctl, const char *uri) {
     NSString *str = [NSString stringWithUTF8String:uri];
-    NSURL *url = [NSURL URLWithString:str];
-    if (!url) {
+
+    NSURL *url;
+
+    if (uri[0] == '/') {
         url = [NSURL fileURLWithPath:str];
     }
+    else if (!strncmp(uri, "file://", 7)) {
+        url = [NSURL URLWithString:str];
+    }
+
     if (!url) {
         return -1;
     }
     if (deadbeef->conf_get_int ("cocoaui.delete_use_bin", 1)) {
-        [NSWorkspace.sharedWorkspace recycleURLs:@[url] completionHandler:^(NSDictionary<NSURL *,NSURL *> * _Nonnull newURLs, NSError * _Nullable error) {
-            //            trace ("Failed to delete file: %s\n", uri);
-        }];
+        [NSWorkspace.sharedWorkspace recycleURLs:@[url] completionHandler:nil];
     } else {
         (void)unlink (uri);
 
@@ -450,7 +458,12 @@ _deleteCompleted (ddbDeleteFromDiskController_t ctl, int cancelled) {
 #endif
 
         [alert beginSheetModalForWindow:self.view.window completionHandler:^(NSModalResponse returnCode) {
-            callback(ctl, returnCode != NSAlertSecondButtonReturn);
+            BOOL shouldCancel = returnCode != NSAlertSecondButtonReturn;
+            // Defer the execution to next tick,
+            // to ensure that the alert is dismissed before proceeding.
+            dispatch_async(dispatch_get_main_queue(), ^{
+                callback(ctl, shouldCancel);
+            });
         }];
     }
     else {
@@ -504,6 +517,25 @@ _deleteCompleted (ddbDeleteFromDiskController_t ctl, int cancelled) {
     deadbeef->plt_unref (plt);
 }
 
+- (void)showInFinder {
+    NSMutableArray *urls = [NSMutableArray arrayWithCapacity:ddbUtilTrackListGetTrackCount(self.selectedTracksList)];
+    [self forEachTrack:^(DB_playItem_t *it) {
+        if (deadbeef->pl_is_selected (it)) {
+            const char *uri = deadbeef->pl_find_meta (it, ":URI");
+            NSString *str = [NSString stringWithUTF8String:uri];
+            NSURL *url = [NSURL URLWithString:str];
+            if (!url) {
+                url = [NSURL fileURLWithPath:str];
+            }
+            if (url) {
+                [urls addObject:url];
+            }
+        }
+        return YES;
+    }];
+    [NSWorkspace.sharedWorkspace activateFileViewerSelectingURLs:urls];
+}
+
 #pragma mark - Playback Queue
 
 - (void)addToFrontOfPlaybackQueue {
