@@ -1,6 +1,6 @@
 /*
     DeaDBeeF - The Ultimate Music Player
-    Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>
+    Copyright (C) 2009-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -492,7 +492,7 @@ ffmpeg_seek_sample (DB_fileinfo_t *_info, int sample) {
 
 static int
 ffmpeg_seek (DB_fileinfo_t *_info, float time) {
-    return ffmpeg_seek_sample64 (_info, time * _info->fmt.samplerate);
+    return ffmpeg_seek_sample64 (_info, (int64_t)((double)time * (int64_t)_info->fmt.samplerate));
 }
 
 static const char *map[] = {
@@ -802,7 +802,12 @@ ffmpeg_init_exts (void) {
          * encoding purpose, because ffmpeg will guess the output format from
          * the file name specified by users.
          */
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+        void *iter = NULL;
+        while ((ifmt = av_demuxer_iterate(&iter))) {
+#else
         while ((ifmt = av_iformat_next(ifmt))) {
+#endif
 #ifdef AV_IS_INPUT_DEVICE
             if (ifmt->priv_class && AV_IS_INPUT_DEVICE(ifmt->priv_class->category))
                 continue; // Skip all input devices
@@ -849,7 +854,9 @@ ffmpeg_message (uint32_t id, uintptr_t ctx, uint32_t p1, uint32_t p2) {
 static int
 ffmpeg_start (void) {
     ffmpeg_init_exts ();
+#if LIBAVFORMAT_VERSION_MAJOR < 58
     av_register_all ();
+#endif
     return 0;
 }
 
@@ -922,11 +929,12 @@ static ddb_decoder2_t plugin = {
     .decoder.plugin.version_major = 1,
     .decoder.plugin.version_minor = 2,
     .decoder.plugin.type = DB_PLUGIN_DECODER,
+    .decoder.plugin.flags = DDB_PLUGIN_FLAG_IMPLEMENTS_DECODER2,
     .decoder.plugin.id = "ffmpeg",
     .decoder.plugin.name = "FFMPEG audio player",
     .decoder.plugin.descr = "decodes audio formats using FFMPEG libavcodec",
     .decoder.plugin.copyright =
-        "Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (C) 2009-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "\n"
         "This program is free software; you can redistribute it and/or\n"
         "modify it under the terms of the GNU General Public License\n"
