@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2015 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2015 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -1485,3 +1485,25 @@ on_dsp_remove_toolbtn_clicked          (GtkToolButton   *toolbutton,
 void
 on_dsp_add_toolbtn_toggled             (GtkToggleToolButton *toggletoolbutton,
                                         gpointer         user_data);
+
+void
+on_sortcancel_clicked                  (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_sortok_clicked                      (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_sortfmt_show                        (GtkWidget       *widget,
+                                        gpointer         user_data);
+
+void
+on_use_visualization_background_color_toggled
+                                        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_visualization_custom_background_color_button_color_set
+                                        (GtkColorButton  *colorbutton,
+                                        gpointer         user_data);
