@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2015 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2015 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -68,6 +68,7 @@
 #include "wingeom.h"
 #include "plmenu.h"
 #include "covermanager/albumartwidget.h"
+#include "selpropertieswidget.h"
 
 #define USE_GTK_APPLICATION 1
 
@@ -172,7 +173,7 @@ update_songinfo (gpointer unused) {
     char totaltime_str[512] = "";
     format_timestr(totaltime_str, sizeof (totaltime_str), pl_totaltime);
 
-    DB_playItem_t *track = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *track = deadbeef->streamer_get_playing_track_safe ();
 
     ddb_tf_context_t ctx = {
         ._size = sizeof (ddb_tf_context_t),
@@ -433,7 +434,7 @@ set_title_cb (gpointer data) {
 void
 gtkui_set_titlebar (DB_playItem_t *it) {
     if (!it) {
-        it = deadbeef->streamer_get_playing_track ();
+        it = deadbeef->streamer_get_playing_track_safe ();
     }
     else {
         deadbeef->pl_item_ref (it);
@@ -464,7 +465,7 @@ gtkui_set_titlebar (DB_playItem_t *it) {
 static gboolean
 trackinfochanged_cb (gpointer data) {
     DB_playItem_t *track = data;
-    DB_playItem_t *curr = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *curr = deadbeef->streamer_get_playing_track_safe ();
     if (track == curr) {
         gtkui_set_titlebar (track);
     }
@@ -596,6 +597,39 @@ gtkui_rename_playlist (ddb_playlist_t *plt) {
     return 0;
 }
 
+int gtkui_remove_playlist (ddb_playlist_t *plt) {
+    int plt_idx = deadbeef->plt_get_idx(plt);
+    if (plt_idx == -1) {
+        return -1;
+    }
+
+    char title[500];
+    if (deadbeef->plt_get_first(plt, PL_MAIN) != NULL) {
+        // playlist not empty, confirm first.
+        deadbeef->plt_get_title (plt, title, sizeof (title));
+        GtkWidget *dlg = gtk_message_dialog_new (GTK_WINDOW (mainwin), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, _("Removing playlist"));
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), _("Do you really want to remove the playlist '%s'?"), title);
+        gtk_window_set_title (GTK_WINDOW (dlg), _("Warning"));
+        int response = gtk_dialog_run (GTK_DIALOG (dlg));
+        gtk_widget_destroy (dlg);
+        if (response != GTK_RESPONSE_YES) {
+            return -1;
+        }
+    }
+
+    deadbeef->plt_remove (plt_idx);
+    return 0;
+}
+
+int gtkui_remove_playlist_at_index (int plt_idx) {
+    int res = -1;
+    ddb_playlist_t *p = deadbeef->plt_get_for_idx (plt_idx);
+    if (p != NULL) {
+        res = gtkui_remove_playlist (p);
+    }
+    return res;
+}
+
 int
 gtkui_rename_playlist_at_index (int plt_idx) {
     int res = -1;
@@ -834,13 +868,11 @@ pre_songstarted_cb (gpointer data) {
 
 static gboolean
 pre_trackfocus_cb (gpointer data) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
         playlist_set_intended_scroll (it);
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return FALSE;
 }
 
@@ -879,18 +911,18 @@ songstarted_cb (gpointer data) {
 
 static gboolean
 trackfocus_cb (gpointer data) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
+        deadbeef->pl_lock ();
         ddb_playlist_t *plt = deadbeef->pl_get_playlist (it);
         if (plt) {
             deadbeef->plt_set_curr (plt);
             playlist_set_cursor (plt, it);
             deadbeef->plt_unref (plt);
         }
+        deadbeef->pl_unlock ();
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return FALSE;
 }
 
@@ -1424,7 +1456,7 @@ gtkui_mainwin_init(void) {
     w_reg_widget (NULL, 0, w_placeholder_create, "placeholder", NULL);
     w_reg_widget (_("Tabs"), DDB_WF_SUPPORTS_EXTENDED_API, w_tabs_create, "tabs", NULL);
     w_reg_widget (_("Playlist tabs"), 0, w_tabstrip_create, "tabstrip", NULL);
-    w_reg_widget (_("Selection properties"), 0, w_selproperties_create, "selproperties", NULL);
+    w_reg_widget (_("Selection properties"), DDB_WF_SUPPORTS_EXTENDED_API, w_selproperties_create, "selproperties", NULL);
     w_reg_widget (_("Album art display"), DDB_WF_SUPPORTS_EXTENDED_API, w_albumart_create, "coverart", NULL);
     w_reg_widget (_("Oscilloscope"), DDB_WF_SUPPORTS_EXTENDED_API, w_scope_create, "scope", NULL);
     w_reg_widget (_("Spectrum"), DDB_WF_SUPPORTS_EXTENDED_API, w_spectrum_create, "spectrum", NULL);
@@ -2240,7 +2272,7 @@ ddb_gtkui_t plugin = {
 #endif
     .gui.plugin.copyright =
         "GTK+ user interface for DeaDBeeF Player.\n"
-        "Copyright (C) 2009-2015 Alexey Yakovenko and other contributors\n"
+        "Copyright (C) 2009-2015 Oleksiy Yakovenko and other contributors\n"
         "\n"
         "This software is provided 'as-is', without any express or implied\n"
         "warranty.  In no event will the authors be held liable for any damages\n"
