@@ -1,7 +1,7 @@
 /*
     Hotkeys plugin for DeaDBeeF
     Copyright (C) 2009-2011 Viktor Semykin <thesame.ml@gmail.com>
-    Copyright (C) 2012-2013 Alexey Yakovenko <waker@users.sourceforge.net>
+    Copyright (C) 2012-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -584,6 +584,9 @@ hotkeys_reset (void) {
 #ifndef NO_XLIB_H
     need_reset = 1;
     trace ("hotkeys: reset flagged\n");
+#else
+    // When not using X11, it's assumed that this is always called from UI branch.
+    read_config();
 #endif
 }
 
@@ -596,7 +599,7 @@ action_play_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
         int cur = deadbeef->plt_get_cursor (plt, PL_MAIN);
         if (cur != -1) {
             ddb_playItem_t *it = deadbeef->plt_get_item_for_idx (plt, cur, PL_MAIN);
-            ddb_playItem_t *it_playing = deadbeef->streamer_get_playing_track ();
+            ddb_playItem_t *it_playing = deadbeef->streamer_get_playing_track_safe ();
             if (it) {
                 deadbeef->pl_item_unref (it);
             }
@@ -674,9 +677,9 @@ action_play_random_cb (struct DB_plugin_action_s *action, ddb_action_context_t c
 
 int
 action_seek_5p_forward_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
+        deadbeef->pl_lock ();
         float dur = deadbeef->pl_get_item_duration (it);
         if (dur > 0) {
             float pos = deadbeef->streamer_get_playpos ();
@@ -686,17 +689,17 @@ action_seek_5p_forward_cb (struct DB_plugin_action_s *action, ddb_action_context
             }
             deadbeef->sendmessage (DB_EV_SEEK, 0, (uint32_t)(pos * 1000), 0);
         }
+        deadbeef->pl_unlock ();
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return 0;
 }
 
 int
 action_seek_5p_backward_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
+        deadbeef->pl_lock ();
         float dur = deadbeef->pl_get_item_duration (it);
         if (dur > 0) {
             float pos = deadbeef->streamer_get_playpos ();
@@ -706,17 +709,17 @@ action_seek_5p_backward_cb (struct DB_plugin_action_s *action, ddb_action_contex
             }
             deadbeef->sendmessage (DB_EV_SEEK, 0, (uint32_t)(pos * 1000), 0);
         }
+        deadbeef->pl_unlock ();
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return 0;
 }
 
 int
 action_seek_1p_forward_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
+        deadbeef->pl_lock ();
         float dur = deadbeef->pl_get_item_duration (it);
         if (dur > 0) {
             float pos = deadbeef->streamer_get_playpos ();
@@ -726,17 +729,17 @@ action_seek_1p_forward_cb (struct DB_plugin_action_s *action, ddb_action_context
             }
             deadbeef->sendmessage (DB_EV_SEEK, 0, (uint32_t)(pos * 1000), 0);
         }
+        deadbeef->pl_unlock ();
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return 0;
 }
 
 int
 action_seek_1p_backward_cb (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
+        deadbeef->pl_lock ();
         float dur = deadbeef->pl_get_item_duration (it);
         if (dur > 0) {
             float pos = deadbeef->streamer_get_playpos ();
@@ -746,17 +749,17 @@ action_seek_1p_backward_cb (struct DB_plugin_action_s *action, ddb_action_contex
             }
             deadbeef->sendmessage (DB_EV_SEEK, 0, (uint32_t)(pos * 1000), 0);
         }
+        deadbeef->pl_unlock ();
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return 0;
 }
 
 static int
 seek_sec (float sec) {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (it) {
+        deadbeef->pl_lock ();
         float dur = deadbeef->pl_get_item_duration (it);
         if (dur > 0) {
             float pos = deadbeef->streamer_get_playpos ();
@@ -769,9 +772,9 @@ seek_sec (float sec) {
             }
             deadbeef->sendmessage (DB_EV_SEEK, 0, (uint32_t)(pos * 1000), 0);
         }
+        deadbeef->pl_unlock ();
         deadbeef->pl_item_unref (it);
     }
-    deadbeef->pl_unlock ();
     return 0;
 }
 
@@ -836,7 +839,7 @@ static DB_plugin_action_t action_prev_or_restart = {
 static DB_plugin_action_t action_duplicate_playlist = {
     .title = "Duplicate Playlist",
     .name = "duplicate_playlist",
-    .flags = DB_ACTION_MULTIPLE_TRACKS,
+    .flags = DB_ACTION_PLAYLIST | DB_ACTION_ADD_MENU,
     .callback2 = action_duplicate_playlist_cb,
     .next = &action_prev_or_restart
 };
@@ -1315,7 +1318,7 @@ static DB_hotkeys_plugin_t plugin = {
         "    * added local hotkeys support\n"
     ,
     .misc.plugin.copyright = 
-        "Copyright (C) 2012-2013 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (C) 2012-2013 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "Copyright (C) 2009-2011 Viktor Semykin <thesame.ml@gmail.com>\n"
         "\n"
         "This program is free software; you can redistribute it and/or\n"
