@@ -1,6 +1,6 @@
 /*
  MPEG decoder plugin for DeaDBeeF Player
- Copyright (C) 2009-2014 Alexey Yakovenko
+ Copyright (C) 2009-2014 Oleksiy Yakovenko
 
  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
@@ -75,7 +75,7 @@ typedef struct {
             struct mad_stream mad_stream;
             struct mad_frame mad_frame;
             struct mad_synth mad_synth;
-            int input_remaining_bytes;
+            long input_remaining_bytes;
         };
 #endif
 #ifdef USE_LIBMPG123
