@@ -1,6 +1,6 @@
 /*
     Playlist browser widget plugin for DeaDBeeF Player
-    Copyright (C) 2009-2014 Alexey Yakovenko
+    Copyright (C) 2009-2014 Oleksiy Yakovenko
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -786,8 +786,10 @@ on_pltbrowser_popup_menu (GtkWidget *widget, gpointer user_data) {
 
 static void
 on_pltbrowser_row_activated (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data) {
-    if (deadbeef->conf_get_int ("gtkui.pltbrowser.play_on_double_click", 1))
-        deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, 0, 0);
+    if (deadbeef->conf_get_int ("gtkui.pltbrowser.play_on_double_click", 1)) {
+        deadbeef->sendmessage (DB_EV_STOP, 0, 0, 0);
+        deadbeef->sendmessage (DB_EV_NEXT, 0, 0, 0);
+    }
 }
 
 static void
@@ -939,7 +941,7 @@ static DB_misc_t plugin = {
     .plugin.descr = "Use View -> Design Mode to add playlist browser into main window",
     .plugin.copyright = 
         "Playlist browser widget plugin for DeaDBeeF Player\n"
-        "Copyright (C) 2009-2014 Alexey Yakovenko\n"
+        "Copyright (C) 2009-2014 Oleksiy Yakovenko\n"
         "\n"
         "This software is provided 'as-is', without any express or implied\n"
         "warranty.  In no event will the authors be held liable for any damages\n"
