@@ -2,7 +2,7 @@
     Shellexec plugin for DeaDBeeF
     Copyright (C) 2010-2014 Deadbeef team
     Original developer Viktor Semykin <thesame.ml@gmail.com>
-    Maintenance, minor improvements Alexey Yakovenko <waker@users.sf.net>
+    Maintenance, minor improvements Oleksiy Yakovenko <waker@users.sf.net>
     GUI support and bugfixing Azeem Arshad <kr00r4n@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
@@ -153,7 +153,7 @@ shx_callback (Shx_action_t *action, int ctx)
         break;
     case DDB_ACTION_CTX_NOWPLAYING:
         {
-            DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+            DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
             if (it) {
                 res = shx_exec_track_cmd (action, it);
                 deadbeef->pl_item_unref (it);
@@ -439,7 +439,7 @@ static Shx_plugin_t plugin = {
         "Shellexec plugin for DeaDBeeF\n"
         "Copyright (C) 2010-2014 Deadbeef team\n"
         "Original developer Viktor Semykin <thesame.ml@gmail.com>\n"
-        "Maintenance, minor improvements Alexey Yakovenko <waker@users.sf.net>\n"
+        "Maintenance, minor improvements Oleksiy Yakovenko <waker@users.sf.net>\n"
         "GUI support and bugfixing Azeem Arshad <kr00r4n@gmail.com>"
         "\n"
         "This program is free software; you can redistribute it and/or\n"
