@@ -1,6 +1,6 @@
 /*
     SHN (Shorten) plugin for DeaDBeeF Player
-    Copyright (C) 2009-2014 Alexey Yakovenko <waker@users.sourceforge.net>
+    Copyright (C) 2009-2014 Oleksiy Yakovenko <waker@users.sourceforge.net>
 
     Based on xmms-shn, http://www.etree.org/shnutils/xmms-shn/
     Copyright (C) 2000-2007  Jason Jordan <shnutils@freeshell.org>
@@ -1174,8 +1174,8 @@ void fwrite_type(slong **data,int ftype,int nchan,int nitem,shn_file *this_shn)
 /* bitshifts */
 /*************/
 
-void fix_bitshift(buffer, nitem, bitshift, ftype) slong *buffer; int nitem,
-       bitshift, ftype; {
+void fix_bitshift(slong *buffer, int nitem, int bitshift, int ftype)
+{
   int i;
 
   if(ftype == TYPE_AU1)
@@ -1817,7 +1817,7 @@ static DB_decoder_t plugin = {
     .plugin.descr = "decodes shn files",
     .plugin.copyright = 
         "SHN (Shorten) plugin for DeaDBeeF Player\n"
-        "Copyright (C) 2009-2014 Alexey Yakovenko <waker@users.sourceforge.net>\n"
+        "Copyright (C) 2009-2014 Oleksiy Yakovenko <waker@users.sourceforge.net>\n"
         "\n"
         "Based on xmms-shn, http://www.etree.org/shnutils/xmms-shn/\n"
         "Copyright (C) 2000-2007  Jason Jordan <shnutils@freeshell.org>\n"
