@@ -4,7 +4,7 @@
 
   routines for converting between wave formats and channel configurations
 
-  Copyright (C) 2009-2013 Alexey Yakovenko
+  Copyright (C) 2009-2013 Oleksiy Yakovenko
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 
-  Alexey Yakovenko waker@users.sourceforge.net
+  Oleksiy Yakovenko waker@users.sourceforge.net
 */
 
 #ifndef __PREMIX_H
@@ -30,8 +30,17 @@
 
 #include "deadbeef.h"
 
+#ifdef __cplusplus
+#define restrict
+extern "C" {
+#endif
+
 // @returns number of output bytes
 int
 pcm_convert (const ddb_waveformat_t * restrict inputfmt, const char * restrict input, const ddb_waveformat_t * restrict outputfmt, char * restrict output, int inputsize);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
