@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2016 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2016 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -25,15 +25,27 @@
 #define mp4tagutil_h
 
 #include "../deadbeef.h"
-#include "mp4ff.h"
+#include <mp4p/mp4p.h>
 
 int
 mp4_read_metadata (DB_playItem_t *it);
 
 int
-mp4_read_metadata_file (DB_playItem_t *it, DB_FILE *fp);
+mp4_read_metadata_file (DB_playItem_t *it, mp4p_file_callbacks_t *cb);
 
 int
 mp4_write_metadata (DB_playItem_t *it);
 
+mp4p_atom_t *
+mp4tagutil_modify_meta (mp4p_atom_t *mp4file, DB_playItem_t *it);
+
+void
+mp4_init_ddb_file_callbacks (mp4p_file_callbacks_t *cb);
+
+void
+mp4_load_tags (mp4p_atom_t *mp4file, DB_playItem_t *it);
+
+mp4p_atom_t *
+mp4_get_cover_atom (mp4p_atom_t *mp4file);
+
 #endif /* mp4tagutil_h */
