@@ -4,7 +4,7 @@
 
   streamer implementation
 
-  Copyright (C) 2009-2017 Alexey Yakovenko
+  Copyright (C) 2009-2017 Oleksiy Yakovenko
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 
-  Alexey Yakovenko waker@users.sourceforge.net
+  Oleksiy Yakovenko waker@users.sourceforge.net
 */
 #include <stdlib.h>
 #include <string.h>
@@ -43,7 +43,6 @@
 #include "messagepump.h"
 #include "conf.h"
 #include "plugins.h"
-#include "fastftoi.h"
 #include "volume.h"
 #include "vfs.h"
 #include "premix.h"
@@ -106,6 +105,11 @@ static int streamer_is_buffering;
 
 static playlist_t *streamer_playlist;
 static playItem_t *playing_track;
+
+// If streaming_track has been removed from playlist, fallback to one of those tracks to pick next/prev.
+static playItem_t *next_track_to_play;
+static playItem_t *prev_track_to_play;
+
 static playItem_t *buffering_track;
 static float playtime; // total playtime of playing track
 static time_t started_timestamp; // result of calling time(NULL)
@@ -197,6 +201,7 @@ _streamer_mark_album_played_up_to (playItem_t *item);
 
 static void
 streamer_abort_files (void) {
+    streamer_lock ();
     DB_vfs_t *file_vfs = fileinfo_file_vfs;
     uint64_t file_identifier = fileinfo_file_identifier;
 
@@ -205,6 +210,7 @@ streamer_abort_files (void) {
 
     DB_vfs_t *strfile_vfs =streamer_file_vfs;
     uint64_t strfile_identifier = streamer_file_identifier;
+    streamer_unlock();
 
     trace ("\033[0;33mstreamer_abort_files\033[37;0m\n");
     trace ("%lld %lld %lld\n", file_identifier, newfile_identifier, strfile_identifier);
@@ -259,6 +265,7 @@ send_trackchanged (playItem_t *from, playItem_t *to) {
 
 void
 streamer_set_last_played (playItem_t *track) {
+    streamer_lock();
     if (last_played) {
         pl_item_unref (last_played);
     }
@@ -266,6 +273,7 @@ streamer_set_last_played (playItem_t *track) {
     if (last_played) {
         pl_item_ref (last_played);
     }
+    streamer_unlock();
 }
 
 static void
@@ -286,6 +294,15 @@ streamer_start_playback (playItem_t *from, playItem_t *it) {
             playqueue_pop ();
         }
         if (qnext) {
+            // playlist change?
+            int idx = plt_get_item_idx(streamer_playlist, qnext, PL_MAIN);
+            if (idx == -1) {
+                playlist_t *plt = pl_get_playlist(qnext);
+                if (plt != NULL) {
+                    streamer_set_streamer_playlist (plt);
+                    plt_unref (plt);
+                }
+            }
             pl_item_unref (qnext);
         }
 
@@ -302,6 +319,20 @@ streamer_start_playback (playItem_t *from, playItem_t *it) {
     trace ("streamer_start_playback %s\n", playing_track ? pl_find_meta (playing_track, ":URI") : "null");
 }
 
+
+static void
+streamer_set_streaming_track(playItem_t *it) {
+    streamer_lock();
+    if (streaming_track) {
+        pl_item_unref (streaming_track);
+    }
+    streaming_track = it;
+    if (streaming_track) {
+        pl_item_ref (streaming_track);
+    }
+    streamer_unlock();
+}
+
 playItem_t *
 streamer_get_streaming_track (void) {
     if (streaming_track) {
@@ -311,7 +342,7 @@ streamer_get_streaming_track (void) {
 }
 
 playItem_t *
-streamer_get_playing_track (void) {
+streamer_get_playing_track_unsafe (void) {
     playItem_t *it = (buffering_track && !playing_track) ? buffering_track : playing_track;
     if (it) {
         pl_item_ref (it);
@@ -319,6 +350,44 @@ streamer_get_playing_track (void) {
     return it;
 }
 
+playItem_t *
+streamer_get_playing_track (void) {
+    // some plugins may call this from plugin.start, before streamer is initialized
+    if (mutex == 0) {
+        return NULL;
+    }
+    streamer_lock();
+    playItem_t *it = streamer_get_playing_track_unsafe();
+    streamer_unlock();
+    return it;
+}
+
+
+void
+streamer_set_playing_track (playItem_t *it) {
+    if (it == playing_track) {
+        return;
+    }
+
+    streamer_lock();
+    playItem_t *prev = playing_track;
+    playing_track = it;
+    streamer_unlock();
+
+    if (it) {
+        pl_item_ref (it);
+    }
+
+    if (prev) {
+        send_trackinfochanged(prev);
+        pl_item_unref (prev);
+    }
+
+    if (it) {
+        send_trackinfochanged(it);
+    }
+}
+
 playItem_t *
 streamer_get_buffering_track (void) {
     playItem_t *it = buffering_track;
@@ -328,6 +397,33 @@ streamer_get_buffering_track (void) {
     return it;
 }
 
+void
+streamer_set_buffering_track (playItem_t *it) {
+    streamer_lock();
+    if (it == buffering_track) {
+        streamer_unlock();
+        return;
+    }
+
+    playItem_t *prev = buffering_track;
+
+    buffering_track = it;
+    if (buffering_track) {
+        pl_item_ref (buffering_track);
+    }
+    streamer_unlock();
+
+    send_trackinfochanged(prev);
+
+    if (it) {
+        send_trackinfochanged(it);
+    }
+
+    if (prev) {
+        pl_item_unref (prev);
+    }
+}
+
 int
 str_get_idx_of (playItem_t *it) {
     pl_lock ();
@@ -460,6 +556,13 @@ get_random_track (void) {
 static playItem_t *
 get_next_track (playItem_t *curr, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
     pl_lock ();
+
+    if (next_track_to_play != NULL) {
+        pl_item_ref(next_track_to_play);
+        pl_unlock();
+        return next_track_to_play;
+    }
+
     if (!streamer_playlist) {
         playlist_t *plt = plt_get_curr ();
         streamer_set_streamer_playlist (plt);
@@ -481,12 +584,14 @@ get_next_track (playItem_t *curr, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
         return NULL; // empty playlist
     }
 
-    playlist_t *item_plt = pl_get_playlist(curr);
-    if (!item_plt) {
-        curr = NULL;
-    }
-    if (item_plt) {
-        plt_unref (item_plt);
+    if (plt_get_item_idx (streamer_playlist, curr, PL_MAIN) == -1) {
+        playlist_t *item_plt = pl_get_playlist(curr);
+        if (!item_plt) {
+            curr = NULL;
+        }
+        if (item_plt) {
+            plt_unref (item_plt);
+        }
     }
 
     if (shuffle == DDB_SHUFFLE_TRACKS || shuffle == DDB_SHUFFLE_ALBUMS) { // shuffle
@@ -495,10 +600,10 @@ get_next_track (playItem_t *curr, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
             // find minimal notplayed
             playItem_t *pmin = NULL; // notplayed minimum
             for (playItem_t *i = plt->head[PL_MAIN]; i; i = i->next[PL_MAIN]) {
-                if (i->played) {
+                if (pl_get_played (i)) {
                     continue;
                 }
-                if (!pmin || i->shufflerating < pmin->shufflerating) {
+                if (!pmin || pl_get_shufflerating(i) < pl_get_shufflerating(pmin)) {
                     pmin = i;
                 }
             }
@@ -516,13 +621,13 @@ get_next_track (playItem_t *curr, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
         }
         else {
             // find minimal notplayed above current
-            int rating = curr->shufflerating;
+            int rating = pl_get_shufflerating(curr);
             playItem_t *pmin = NULL; // notplayed minimum
             for (playItem_t *i = plt->head[PL_MAIN]; i; i = i->next[PL_MAIN]) {
-                if (i->played || i->shufflerating < rating) {
+                if (pl_get_played(i) || pl_get_shufflerating (i) < rating) {
                     continue;
                 }
-                if (!pmin || i->shufflerating < pmin->shufflerating) {
+                if (!pmin || pl_get_shufflerating (i) < pl_get_shufflerating (pmin)) {
                     pmin = i;
                 }
             }
@@ -594,6 +699,12 @@ static playItem_t *
 get_prev_track (playItem_t *curr, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
     pl_lock ();
     
+    if (prev_track_to_play != NULL) {
+        pl_item_ref(prev_track_to_play);
+        pl_unlock();
+        return prev_track_to_play;
+    }
+
     // check if prev song is in this playlist
     if (curr && -1 == str_get_idx_of (curr)) {
         curr = NULL;
@@ -619,25 +730,26 @@ get_prev_track (playItem_t *curr, ddb_shuffle_t shuffle, ddb_repeat_t repeat) {
             return it;
         }
         else {
-            curr->played = 0;
+            pl_set_played(curr, 0);
             // find already played song with maximum shuffle rating below prev song
-            int rating = curr->shufflerating;
+            int rating = pl_get_shufflerating(curr);
             playItem_t *pmax = NULL; // played maximum
             playItem_t *amax = NULL; // absolute maximum
             for (playItem_t *i = plt->head[PL_MAIN]; i; i = i->next[PL_MAIN]) {
-                if (i != curr && i->played && (!amax || i->shufflerating > amax->shufflerating)) {
+                int played = pl_get_played(i);
+                if (i != curr && played && (!amax || pl_get_shufflerating (i) > pl_get_shufflerating (amax))) {
                     amax = i;
                 }
-                if (i == curr || i->shufflerating > rating || !i->played) {
+                if (i == curr || pl_get_shufflerating (i) > rating || !played) {
                     continue;
                 }
-                if (!pmax || i->shufflerating > pmax->shufflerating) {
+                if (!pmax || pl_get_shufflerating (i) > pl_get_shufflerating (pmax)) {
                     pmax = i;
                 }
             }
 
             if (pmax && shuffle == DDB_SHUFFLE_ALBUMS) {
-                while (pmax && pmax->next[PL_MAIN] && pmax->next[PL_MAIN]->played && pmax->shufflerating == pmax->next[PL_MAIN]->shufflerating) {
+                while (pmax && pmax->next[PL_MAIN] && pl_get_played(pmax->next[PL_MAIN]) && pl_get_shufflerating (pmax) == pl_get_shufflerating ( pmax->next[PL_MAIN])) {
                     pmax = pmax->next[PL_MAIN];
                 }
             }
@@ -713,12 +825,59 @@ streamer_move_to_randomsong (int r) {
     return 0;
 }
 
+static void
+streamer_set_next_track_to_play(playItem_t *next) {
+    if (next_track_to_play != NULL) {
+        pl_item_unref(next_track_to_play);
+        next_track_to_play = NULL;
+    }
+    next_track_to_play = next;
+    if (next_track_to_play) {
+        pl_item_ref(next_track_to_play);
+    }
+}
+
+static void
+streamer_set_prev_track_to_play(playItem_t *prev) {
+    if (prev_track_to_play != NULL) {
+        pl_item_unref(prev_track_to_play);
+        prev_track_to_play = NULL;
+    }
+    prev_track_to_play = prev;
+    if (prev_track_to_play) {
+        pl_item_ref(prev_track_to_play);
+    }
+}
+
 // playlist must call that whenever item was removed
 void
 streamer_song_removed_notify (playItem_t *it) {
     if (!mutex) {
         return; // streamer is not running
     }
+    streamer_lock();
+
+    playItem_t *next = NULL;
+    playItem_t *prev = NULL;
+
+    if (streaming_track == it || next_track_to_play == it) {
+        ddb_shuffle_t shuffle = streamer_get_shuffle ();
+        ddb_repeat_t repeat = streamer_get_repeat ();
+        streamer_set_next_track_to_play (NULL);
+        next = get_next_track (it, shuffle, repeat);
+        prev = get_prev_track (it, shuffle, repeat);
+    }
+    streamer_set_next_track_to_play (next);
+    streamer_set_prev_track_to_play (prev);
+
+    if (next != NULL) {
+        pl_item_unref(next);
+    }
+    if (prev != NULL) {
+        pl_item_unref(prev);
+    }
+
+    streamer_unlock();
 }
 
 static ddb_ctmap_t *streamer_ctmap;
@@ -801,13 +960,7 @@ streamer_play_failed (playItem_t *failed_track) {
         new_fileinfo_file_vfs = NULL;
         new_fileinfo_file_identifier = 0;
 
-        if (streaming_track) {
-            pl_item_unref (streaming_track);
-        }
-        streaming_track = failed_track;
-        if (streaming_track) {
-            pl_item_ref (streaming_track);
-        }
+        streamer_set_streaming_track(failed_track);
     }
     streamer_unlock();
 }
@@ -824,12 +977,14 @@ fileinfo_free (DB_fileinfo_t *fileinfo) {
 
 static int
 stream_track (playItem_t *it, int startpaused) {
+    streamer_lock();
     if (fileinfo_curr) {
         fileinfo_free (fileinfo_curr);
         fileinfo_curr = NULL;
         fileinfo_file_vfs = NULL;
         fileinfo_file_identifier = 0;
     }
+    streamer_unlock();
     trace ("stream_track %s\n", playing_track ? pl_find_meta (playing_track, ":URI") : "null");
     int err = 0;
     playItem_t *from = NULL;
@@ -848,16 +1003,11 @@ stream_track (playItem_t *it, int startpaused) {
         pl_item_ref (from);
     }
     if (to) {
-        to->played = 1;
+        pl_set_played(to, 1);
         pl_item_ref (to);
     }
 
-    streamer_lock ();
-    if (streaming_track) {
-        pl_item_unref (streaming_track);
-        streaming_track = NULL;
-    }
-    streamer_unlock ();
+    streamer_set_streaming_track(NULL);
 
     int paused_stream = 0;
     if (it && startpaused) {
@@ -1098,7 +1248,7 @@ m3u_error:
     int plug_idx = 0;
     for (;;) {
         if (!decoder_id[0] && plugs[0] && !plugs[plug_idx]) {
-            it->played = 1;
+            pl_set_played(it, 1);
             trace_err ("No suitable decoder found for stream %s of content-type %s\n", pl_find_meta (playing_track, ":URI"), cct);
 
             if (!startpaused) {
@@ -1168,6 +1318,7 @@ m3u_error:
         }
 
         trace ("\033[0;33minit decoder for %s (%s)\033[37;0m\n", pl_find_meta (it, ":URI"), dec->plugin.id);
+        streamer_lock();
         new_fileinfo = dec_open (dec, STREAMER_HINTS, it);
         if (new_fileinfo && new_fileinfo->file) {
             new_fileinfo_file_vfs = new_fileinfo->file->vfs;
@@ -1177,20 +1328,26 @@ m3u_error:
             new_fileinfo_file_vfs = NULL;
             new_fileinfo_file_identifier = 0;
         }
-        if (new_fileinfo && dec->init (new_fileinfo, DB_PLAYITEM (it)) != 0) {
+        int have_new_fileinfo = new_fileinfo != NULL;
+        streamer_unlock();
+        if (have_new_fileinfo && dec->init (new_fileinfo, DB_PLAYITEM (it)) != 0) {
             trace ("\033[0;31mfailed to init decoder\033[37;0m\n");
             pl_delete_meta (it, "!DECODER");
+            streamer_lock();
             dec->free (new_fileinfo);
             new_fileinfo = NULL;
             new_fileinfo_file_vfs = NULL;
             new_fileinfo_file_identifier = 0;
+            have_new_fileinfo = 0;
+            streamer_unlock();
         }
 
-        if (!new_fileinfo) {
+        if (!have_new_fileinfo) {
             trace ("decoder %s failed\n", dec->plugin.id);
             continue;
         }
         else {
+            streamer_lock();
             if (new_fileinfo->file) {
                 new_fileinfo_file_vfs = new_fileinfo->file->vfs;
                 new_fileinfo_file_identifier = vfs_get_identifier (new_fileinfo->file);
@@ -1200,13 +1357,8 @@ m3u_error:
                 new_fileinfo_file_identifier = 0;
             }
 
-            if (streaming_track) {
-                pl_item_unref (streaming_track);
-            }
-            streaming_track = it;
-            if (streaming_track) {
-                pl_item_ref (streaming_track);
-            }
+            streamer_set_streaming_track (it);
+            streamer_unlock();
 
             trace ("bps=%d, channels=%d, samplerate=%d\n", new_fileinfo->fmt.bps, new_fileinfo->fmt.channels, new_fileinfo->fmt.samplerate);
             break;
@@ -1214,12 +1366,14 @@ m3u_error:
     }
 success:
     streamer_play_failed (NULL);
+    streamer_lock();
     if (new_fileinfo) {
         fileinfo_curr = new_fileinfo;
         new_fileinfo = NULL;
         new_fileinfo_file_vfs = NULL;
         new_fileinfo_file_identifier = 0;
     }
+    streamer_unlock();
 
 error:
     if (from) {
@@ -1234,16 +1388,23 @@ error:
 
 float
 streamer_get_playpos (void) {
+    streamer_lock();
     float seek = last_seekpos;
     if (seek >= 0) {
+        streamer_unlock();
         return seek;
     }
-    return playpos;
+    float ret = playpos;
+    streamer_unlock();
+    return ret;
 }
 
 int
 streamer_get_apx_bitrate (void) {
-    return avg_bitrate;
+    streamer_lock ();
+    int res = avg_bitrate;
+    streamer_unlock ();
+    return res;
 }
 
 void
@@ -1313,7 +1474,10 @@ streamer_seek_real (float seekpos) {
         if (seek >= dur) {
             seek = dur - 0.000001f;
         }
+
+        streamer_lock ();
         playpos = seek;
+        streamer_unlock ();
         trace ("seeking to %f\n", seek);
 
         if (track == playing_track && track != streaming_track) {
@@ -1341,16 +1505,22 @@ streamer_seek_real (float seekpos) {
         ev->playpos = playpos;
         messagepump_push_event ((ddb_event_t*)ev, 0, 0);
     }
+    streamer_lock();
     last_seekpos = -1;
+    streamer_unlock();
 }
 
 static void
 _update_buffering_state () {
+    streamer_lock ();
     int blocks_ready = streamreader_num_blocks_ready ();
+    streamer_unlock ();
     int buffering = (blocks_ready < 4) && streaming_track;
 
     if (buffering != streamer_is_buffering) {
+        streamer_lock();
         streamer_is_buffering = buffering;
+        streamer_unlock();
 
         // update buffering UI
         if (!buffering) {
@@ -1376,7 +1546,9 @@ handle_track_change (playItem_t *from, playItem_t *track) {
     // next track started
     if (from) {
         send_songfinished (from);
+        streamer_lock();
         playpos = 0;
+        streamer_unlock();
     }
 
     streamer_start_playback (from, track);
@@ -1459,13 +1631,7 @@ _streamer_requeue_after_current (ddb_repeat_t repeat, ddb_shuffle_t shuffle) {
         return;
     }
 
-    if (streaming_track) {
-        pl_item_unref (streaming_track);
-    }
-    streaming_track = playing_track;
-    if (streaming_track) {
-        pl_item_ref (streaming_track);
-    }
+    streamer_set_streaming_track(playing_track);
     streamer_unlock ();
     streamer_next (shuffle, repeat, NULL);
 }
@@ -1594,7 +1760,9 @@ streamer_thread (void *unused) {
             continue;
         }
 
+        streamer_lock();
         streamblock_t *block = streamreader_get_next_block ();
+        streamer_unlock();
 
         if (!block) {
             usleep (50000); // all blocks are full
@@ -1666,21 +1834,15 @@ streamer_thread (void *unused) {
     while (!handler_pop (handler, &id, &ctx, &p1, &p2));
 
     // stop streaming song
+    streamer_lock ();
     if (fileinfo_curr) {
         fileinfo_free (fileinfo_curr);
         fileinfo_curr = NULL;
         fileinfo_file_vfs = NULL;
         fileinfo_file_identifier = 0;
     }
-    streamer_lock ();
-    if (streaming_track) {
-        pl_item_unref (streaming_track);
-        streaming_track = NULL;
-    }
-    if (playing_track) {
-        pl_item_unref (playing_track);
-        playing_track = NULL;
-    }
+    streamer_set_streaming_track(NULL);
+    streamer_set_playing_track(NULL);
     streamer_unlock ();
 }
 
@@ -1732,22 +1894,12 @@ streamer_free (void) {
         pl_item_unref (first_failed_track);
         first_failed_track = NULL;
     }
-    if (streaming_track) {
-        pl_item_unref (streaming_track);
-        streaming_track = NULL;
-    }
-    if (playing_track) {
-        pl_item_unref (playing_track);
-        playing_track = NULL;
-    }
-    if (buffering_track) {
-        pl_item_unref (buffering_track);
-        buffering_track = NULL;
-    }
-    if (last_played) {
-        pl_item_unref (last_played);
-        last_played = NULL;
-    }
+    streamer_set_streaming_track(NULL);
+    streamer_set_next_track_to_play(NULL);
+    streamer_set_prev_track_to_play(NULL);
+    streamer_set_playing_track(NULL);
+    streamer_set_buffering_track(NULL);
+    streamer_set_last_played(NULL);
     streamer_set_streamer_playlist (NULL);
 
     ddb_ctmap_free (streamer_ctmap);
@@ -1909,6 +2061,7 @@ process_output_block (streamblock_t *block, char *bytes, int bytes_available_siz
         memcpy (bytes, dspbytes, sz);
     }
 
+    streamer_lock();
     decoded_block->track = block->track;
     if (decoded_block->track != NULL) {
         pl_item_ref (decoded_block->track);
@@ -1921,6 +2074,8 @@ process_output_block (streamblock_t *block, char *bytes, int bytes_available_siz
 
     block->pos = block->size;
     streamreader_next_block ();
+    streamer_unlock();
+
     _update_buffering_state ();
 
     return sz;
@@ -1929,6 +2084,7 @@ process_output_block (streamblock_t *block, char *bytes, int bytes_available_siz
 
 static float (*streamer_volume_modifier) (float delta_time);
 
+// used in android branch, do not delete
 void
 streamer_set_volume_modifier (float (*modifier) (float delta_time)) {
     streamer_volume_modifier = modifier;
@@ -1936,7 +2092,14 @@ streamer_set_volume_modifier (float (*modifier) (float delta_time)) {
 
 static void
 streamer_apply_soft_volume (char *bytes, int sz) {
+    if (audio_is_mute ()) {
+        memset (bytes, 0, sz);
+        return;
+    }
     DB_output_t *output = plug_get_output ();
+    if (output->has_volume) {
+        return;
+    }
 
     float mod = 1.f;
 
@@ -1947,66 +2110,58 @@ streamer_apply_soft_volume (char *bytes, int sz) {
 
     float vol = volume_get_amp () * mod;
 
-    if (!output->has_volume) {
-        int mult = 1-audio_is_mute ();
-        char *stream = bytes;
-        int bytesread = sz;
-        if (output->fmt.bps == 16) {
-            mult *= 1000;
-            int16_t ivolume = (int16_t)(vol * mult);
-            if (ivolume != 1000) {
-                int half = bytesread/2;
-                for (int i = 0; i < half; i++) {
-                    int16_t sample = *((int16_t*)stream);
-                    *((int16_t*)stream) = (int16_t)(((int32_t)sample) * ivolume / 1000);
-                    stream += 2;
-                }
+    char *stream = bytes;
+    int bytesread = sz;
+    if (output->fmt.bps == 16) {
+        int16_t ivolume = (int16_t)(vol * 1000);
+        if (ivolume != 1000) {
+            int half = bytesread/2;
+            for (int i = 0; i < half; i++) {
+                int16_t sample = *((int16_t*)stream);
+                *((int16_t*)stream) = (int16_t)(((int32_t)sample) * ivolume / 1000);
+                stream += 2;
             }
         }
-        else if (output->fmt.bps == 8) {
-            mult *= 255;
-            int16_t ivolume = (int16_t)(vol * mult);
-            if (ivolume != 255) {
-                for (int i = 0; i < bytesread; i++) {
-                    *stream = (int8_t)(((int32_t)(*stream)) * ivolume / 1000);
-                    stream++;
-                }
+    }
+    else if (output->fmt.bps == 8) {
+        int16_t ivolume = (int16_t)(vol * 255);
+        if (ivolume != 255) {
+            for (int i = 0; i < bytesread; i++) {
+                *stream = (int8_t)(((int32_t)(*stream)) * ivolume / 255);
+                stream++;
             }
         }
-        else if (output->fmt.bps == 24) {
-            mult *= 1000;
-            int16_t ivolume = (int16_t)(vol * mult);
-            if (ivolume != 1000) {
-                int third = bytesread/3;
-                for (int i = 0; i < third; i++) {
-                    int32_t sample = ((unsigned char)stream[0]) | ((unsigned char)stream[1]<<8) | ((signed char)stream[2]<<16);
-                    int32_t newsample = (int32_t)((int64_t)sample * ivolume / 1000);
-                    stream[0] = (newsample&0x0000ff);
-                    stream[1] = (newsample&0x00ff00)>>8;
-                    stream[2] = (newsample&0xff0000)>>16;
-                    stream += 3;
-                }
+    }
+    else if (output->fmt.bps == 24) {
+        int16_t ivolume = (int16_t)(vol * 1000);
+        if (ivolume != 1000) {
+            int third = bytesread/3;
+            for (int i = 0; i < third; i++) {
+                int32_t sample = ((unsigned char)stream[0]) | ((unsigned char)stream[1]<<8) | ((signed char)stream[2]<<16);
+                int32_t newsample = (int32_t)((int64_t)sample * ivolume / 1000);
+                stream[0] = (newsample&0x0000ff);
+                stream[1] = (newsample&0x00ff00)>>8;
+                stream[2] = (newsample&0xff0000)>>16;
+                stream += 3;
             }
         }
-        else if (output->fmt.bps == 32 && !output->fmt.is_float) {
-            mult *= 1000;
-            int16_t ivolume = (int16_t)(vol * mult);
-            if (ivolume != 1000) {
-                for (int i = 0; i < bytesread/4; i++) {
-                    int32_t sample = *((int32_t*)stream);
-                    int32_t newsample = (int32_t)((int64_t)sample * ivolume / 1000);
-                    *((int32_t*)stream) = newsample;
-                    stream += 4;
-                }
+    }
+    else if (output->fmt.bps == 32 && !output->fmt.is_float) {
+        int16_t ivolume = (int16_t)(vol * 1000);
+        if (ivolume != 1000) {
+            for (int i = 0; i < bytesread/4; i++) {
+                int32_t sample = *((int32_t*)stream);
+                int32_t newsample = (int32_t)((int64_t)sample * ivolume / 1000);
+                *((int32_t*)stream) = newsample;
+                stream += 4;
             }
         }
-        else if (output->fmt.bps == 32 && output->fmt.is_float) {
-            float fvolume = vol * (1-audio_is_mute ());
-            if (fvolume != 1.f) {
-                for (int i = 0; i < bytesread/4; i++) {
-                    *((float*)stream) = (*((float*)stream)) * fvolume;
-                    stream += 4;
-                }
+    }
+    else if (output->fmt.bps == 32 && output->fmt.is_float) {
+        if (vol != 1.f) {
+            for (int i = 0; i < bytesread/4; i++) {
+                *((float*)stream) = (*((float*)stream)) * vol;
+                stream += 4;
             }
         }
     }
@@ -2018,8 +2173,12 @@ _streamer_get_bytes (char *bytes, int size) {
 
     char *outbuffer = _get_output_buffer (OUTPUT_BUFFER_SIZE);
 
+    streamer_lock ();
+    int remaining = _outbuffer_remaining;
+    streamer_unlock ();
+
     // consume decoded data
-    int sz = min (size, _outbuffer_remaining);
+    int sz = min (size, remaining);
     if (!sz) {
         // no data available
         memset (bytes, 0, size);
@@ -2035,6 +2194,9 @@ _streamer_get_bytes (char *bytes, int size) {
     int rb = sz;
     char *readptr = outbuffer;
     char *writeptr = bytes;
+
+    // streamer_reset may clear all blocks, therefore need a lock
+    streamer_lock();
     while (rb > 0) {
         decoded_block_t *decoded_block = decoded_blocks_current();
         if (decoded_block == NULL) {
@@ -2075,8 +2237,12 @@ _streamer_get_bytes (char *bytes, int size) {
     if (sz < _outbuffer_remaining) {
         // FIXME: This is the slowest operation on audio thread, can be optimized with a ring buffer
         memmove (outbuffer, outbuffer + sz, _outbuffer_remaining - sz);
+        _outbuffer_remaining -= sz;
     }
-    _outbuffer_remaining -= sz;
+    else {
+        _outbuffer_remaining = 0;
+    }
+    streamer_unlock();
 
     streamer_apply_soft_volume (bytes, sz);
 
@@ -2142,7 +2308,6 @@ _streamer_fill_playback_buffer(void) {
         streamer_unlock();
         return;
     }
-    streamer_unlock ();
 
     // approximate bitrate
     if (block_bitrate != -1) {
@@ -2165,6 +2330,7 @@ _streamer_fill_playback_buffer(void) {
         }
         //        printf ("apx bitrate: %d (last %d)\n", avg_bitrate, last_bitrate);
     }
+    streamer_unlock ();
 }
 
 int
@@ -2192,7 +2358,10 @@ streamer_read (char *bytes, int size) {
 
 int
 streamer_ok_to_read (int len) {
-    return !streamer_is_buffering;
+    streamer_lock();
+    int res = !streamer_is_buffering;
+    streamer_unlock();
+    return res;
 }
 
 static int
@@ -2218,7 +2387,7 @@ streamer_configchanged (void) {
     streamer_set_repeat (repeat);
 
     if (playing_track) {
-        playing_track->played = 1;
+        pl_set_played(playing_track, 1);
     }
 
     int formatchanged = 0;
@@ -2278,9 +2447,9 @@ streamer_configchanged (void) {
     }
     conf_playback_buffer_size = playback_buffer_size / 1000.f;
 
-    streamer_unlock ();
-
     streamreader_configchanged ();
+
+    streamer_unlock ();
 }
 
 static void
@@ -2302,8 +2471,10 @@ static void
 _play_track (playItem_t *it, int startpaused) {
     DB_output_t *output = plug_get_output();
     output->stop ();
+    streamer_lock();
     streamer_reset(1);
     streamer_is_buffering = 1;
+    streamer_unlock();
 
     playItem_t *prev = playing_track;
     if (prev) {
@@ -2311,6 +2482,8 @@ _play_track (playItem_t *it, int startpaused) {
     }
 
     streamer_set_playing_track(NULL);
+    streamer_set_next_track_to_play(NULL);
+    streamer_set_prev_track_to_play(NULL);
     streamer_set_buffering_track (it);
     handle_track_change (prev, it);
 
@@ -2354,7 +2527,7 @@ _rebuild_shuffle_albums_after_manual_trigger(playlist_t *plt, playItem_t *it) {
     else {
         // This ensures that the manually triggered item becomes first in shuffle queue.
         // It works because shufflerating is generated using rand(), which gives only numbers in the [0..RAND_MAX] range.
-        it->shufflerating = -1;
+        pl_set_shufflerating (it, -1);
     }
 }
 
@@ -2368,22 +2541,21 @@ play_index (int idx, int startpaused) {
 
     playqueue_clear ();
 
+    plt = plt_get_curr ();
+
+    streamer_set_streamer_playlist (plt);
+
     if (idx < 0) {
+        plt_reshuffle (plt, NULL, NULL);
+        streamer_set_last_played(NULL);
         goto error;
     }
 
-    plt = plt_get_curr ();
     it = plt_get_item_for_idx (plt, idx, PL_MAIN);
     if (!it) {
         goto error;
     }
 
-    pl_lock ();
-    if (plt != streamer_playlist) {
-        streamer_set_streamer_playlist (plt);
-    }
-    pl_unlock();
-
     // rebuild shuffle order
     _rebuild_shuffle_albums_after_manual_trigger (plt, it);
 
@@ -2496,7 +2668,7 @@ streamer_get_next_track_with_direction (int dir, ddb_shuffle_t shuffle, ddb_repe
                     playItem_t *it = streamer_playlist->head[PL_MAIN];
                     while (it) {
                         if (it != next) {
-                            it->played = 1;
+                            pl_set_played (it, 1);
                         }
                         it = it->next[PL_MAIN];
                     }
@@ -2600,11 +2772,11 @@ _streamer_mark_album_played_up_to (playItem_t *item) {
     pl_lock ();
     const char *alb = pl_find_meta_raw (item, "album");
     const char *art = pl_find_meta_raw (item, "artist");
-    item->played = 1;
+    pl_set_played(item, 1);
     playItem_t *next = item->prev[PL_MAIN];
     while (next) {
         if (alb == pl_find_meta_raw (next, "album") && art == pl_find_meta_raw (next, "artist")) {
-            next->played = 1;
+            pl_set_played(next, 1);
             next = next->prev[PL_MAIN];
         }
         else {
@@ -2647,56 +2819,6 @@ streamer_get_handler (void) {
     return handler;
 }
 
-void
-streamer_set_playing_track (playItem_t *it) {
-    if (it == playing_track) {
-        return;
-    }
-
-    playItem_t *prev = playing_track;
-
-    playing_track = it;
-    if (playing_track) {
-        pl_item_ref (playing_track);
-    }
-
-    send_trackinfochanged(prev);
-
-    if (playing_track) {
-        send_trackinfochanged(playing_track);
-    }
-
-    if (prev) {
-        pl_item_unref (prev);
-    }
-}
-
-void
-streamer_set_buffering_track (playItem_t *it) {
-    if (it == buffering_track) {
-        return;
-    }
-
-    playItem_t *prev = buffering_track;
-
-    buffering_track = NULL;
-
-    buffering_track = it;
-    if (buffering_track) {
-        pl_item_ref (buffering_track);
-    }
-
-    send_trackinfochanged(prev);
-
-    if (buffering_track) {
-        send_trackinfochanged(buffering_track);
-    }
-
-    if (prev) {
-        pl_item_unref (prev);
-    }
-}
-
 void
 streamer_yield (void) {
     while (handler_hasmessages(handler)) {
@@ -2706,18 +2828,18 @@ streamer_yield (void) {
 
 void
 streamer_set_output (DB_output_t *output) {
-    if (mutex) {
-        streamer_lock ();
-    }
     DB_output_t *prev = plug_get_output ();
     ddb_playback_state_t state = DDB_PLAYBACK_STATE_STOPPED;
-
     ddb_waveformat_t fmt = {0};
     if (prev) {
         state = prev->state ();
         memcpy (&fmt, &prev->fmt, sizeof (ddb_waveformat_t));
         prev->free ();
     }
+
+    if (mutex) {
+        streamer_lock ();
+    }
     plug_set_output (output);
 
     if (fmt.channels) {
