@@ -1,5 +1,5 @@
-/* C code produced by gperf version 3.0.3 */
-/* Command-line: /Applications/Xcode.app/Contents/Developer/Toolchains/XcodeDefault.xctoolchain/usr/bin/gperf -c -t -H u8_uc_hash -N u8_uc_in_word_set u8_uc_map.txt  */
+/* ANSI-C code produced by gperf version 3.0.3 */
+/* Command-line: /Applications/Xcode_12_4.app/Contents/Developer/Toolchains/XcodeDefault.xctoolchain/usr/bin/gperf -E -L ANSI-C -c -t -H u8_uc_hash -N u8_uc_in_word_set u8_uc_map.txt  */
 /* Computed positions: -k'1-2,$' */
 
 #if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
@@ -26,7 +26,7 @@
       && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
       && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
 /* The character set is not based on ISO-646.  */
-error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gnu-gperf@gnu.org>."
+#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gnu-gperf@gnu.org>."
 #endif
 
 #line 1 "u8_uc_map.txt"
@@ -34,12 +34,6 @@ struct u8_uppercase_map_t {
     const char *name;
     const char *upper;
 };
-
-#define TOTAL_KEYWORDS 1006
-#define MIN_WORD_LENGTH 1
-#define MAX_WORD_LENGTH 7
-#define MIN_HASH_VALUE 1
-#define MAX_HASH_VALUE 2780
 /* maximum key range = 2780, duplicates = 0 */
 
 #ifdef __GNUC__
@@ -50,9 +44,7 @@ inline
 #endif
 #endif
 static unsigned int
-u8_uc_hash (str, len)
-     register const char *str;
-     register unsigned int len;
+u8_uc_hash (register const char *str, register unsigned int len)
 {
   static unsigned short asso_values[] =
     {
@@ -100,10 +92,17 @@ u8_uc_hash (str, len)
 }
 
 struct u8_uppercase_map_t *
-u8_uc_in_word_set (str, len)
-     register const char *str;
-     register unsigned int len;
+u8_uc_in_word_set (register const char *str, register unsigned int len)
 {
+  enum
+    {
+      TOTAL_KEYWORDS = 1006,
+      MIN_WORD_LENGTH = 1,
+      MAX_WORD_LENGTH = 7,
+      MIN_HASH_VALUE = 1,
+      MAX_HASH_VALUE = 2780
+    };
+
   static struct u8_uppercase_map_t wordlist[] =
     {
       {""},
